/*
Firmware for a clock using 7-segment-displays made from resistors and
thermochromic paint.
(C) 2010 Jeroen Domburg (jeroen AT spritesmods.com)

This program is free software: you can redistribute it and/or modify
t under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
	    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
			    
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include <avr/pgmspace.h>

/*
Bits used in the 'font'-array map to the lcds like this:
  ---
 | 0 |
1|   |2
  ---
 | 3 |
4|   |5
  ---
   6
*/
static unsigned const char font[] PROGMEM= 
	{0x77, 0x24, 0x5D, 0x6D, 0x2E, 0x6B, 0x7B, 0x25, 0x7F, 0x6F, 00};

//Positions of the bits output to the shift registers. 
static const char bitpos[4][7] PROGMEM= {
	{18, 19, 23, 21, 20, 17, 16}, //rightmost 7-segment display
	{12, 11, 9, 10, 13, 22, 8},
	{1, 2, 14, 3, 4, 15, 5},
	{-1, -1, 6, -1, -1, 7, -1} //leftmost partial-7-segment-display (the 'one')
};

//Convert the given 4 numbers into a long binary string to be fed into 
//the shift registers.
unsigned long charsToShiftVal(char *numbers) {
	unsigned long ret=0;
	int no, segdata, seg;
	for (no=0; no<4; no++) {
		segdata=pgm_read_byte(&font[(int)numbers[no]]);
		for (seg=0; seg<7; seg++) {
			if (segdata&1) {
				if ((char)pgm_read_byte(&bitpos[no][seg])!=-1) ret|=(1L<<pgm_read_byte(&bitpos[no][seg]));
			}
			segdata>>=1;
		}
	}
	return ret;
}
