/*
Firmware for a clock using 7-segment-displays made from resistors and
thermochromic paint.
(C) 2010 Jeroen Domburg (jeroen AT spritesmods.com)

This program is free software: you can redistribute it and/or modify
t under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
	    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
			    
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "io.h"
#include <avr/io.h>
#include <util/delay.h>

#define SHIFTDAT	0
#define SHIFTCLK	1

#define RFIN		6

void ioInit() {
	DDRB=(1<<SHIFTDAT)|(1<<SHIFTCLK);
}


//Shifts out the value to the shift registers.
static void shiftOut(unsigned long val, int delay) {
	int w;
	unsigned long bit;
	for (bit=(1L<<24); bit!=0L; bit>>=1) {
		if (val&bit) PORTB|=(1<<SHIFTDAT); else PORTB&=~(1<<SHIFTDAT);
		_delay_us(2);
		PORTB|=(1<<SHIFTCLK);
		_delay_us(1);
		PORTB&=~(1<<SHIFTCLK);
	}
	_delay_us(2);
	PORTB|=(1<<SHIFTCLK);
	_delay_us(1);
	PORTB&=~(1<<SHIFTCLK);
	if (val==0L) return;
	for (w=0; w<delay; w++) _delay_ms(1);
}

//This shifts out the bits into the shift register to heat up the
//resistors. Because each segment eats approx. 0.25A, we can't heat them
//all up at once, so this routine will heat up only a few at a time.
void shiftOutStaggered(unsigned long val, int len) {
	unsigned long sendThis=0;
	unsigned long bit=1;
	int x;
	int found=0;

//	shiftOut(val); 
//	shiftOut(0);
//	return;

	for (x=0; x<24; x++) {
		bit<<=1L;
		if (val&bit) {
			sendThis|=bit;
			found++;
		}
		if (found==4 || x==23) {
			shiftOut(sendThis, len);
			found=0;
			sendThis=0;
		}
	}
	shiftOut(0, 0);
}

char rfGetVal() {
	if (PIND&(1<<RFIN)) return 1; else return 0;
}
